"""Add column estimation for ProductionScheduleVersionTaskLink

Revision ID: ce7f46f445dc
Revises: e8bc24998b34
Create Date: 2025-07-24 19:27:55.423759

"""

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "ce7f46f445dc"
down_revision = "e8bc24998b34"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table(
        "production_schedule_version_task_link", schema=None
    ) as batch_op:
        batch_op.add_column(sa.Column("estimation", sa.Float(), nullable=True))

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table(
        "production_schedule_version_task_link", schema=None
    ) as batch_op:
        batch_op.drop_column("estimation")

    # ### end Alembic commands ###
