"""add currency field to budgets

Revision ID: 7a16258f2fab
Revises: 8ab98c178903
Create Date: 2025-04-21 22:27:22.703525

"""

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "7a16258f2fab"
down_revision = "8ab98c178903"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("budget", schema=None) as batch_op:
        batch_op.add_column(sa.Column("currency", sa.String(length=3)))
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("budget", schema=None) as batch_op:
        batch_op.drop_column("currency")
    # ### end Alembic commands ###
