"""Add StatusAutomation.import_last_revision

Revision ID: 57222395f2be
Revises: 5b980f0dc365
Create Date: 2024-05-07 01:23:21.597644

"""

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "57222395f2be"
down_revision = "5b980f0dc365"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("status_automation", schema=None) as batch_op:
        batch_op.add_column(
            sa.Column(
                "import_last_revision",
                sa.Boolean(),
                nullable=True,
                default=False,
                server_default=sa.sql.expression.false(),
            )
        )

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("status_automation", schema=None) as batch_op:
        batch_op.drop_column("import_last_revision")

    # ### end Alembic commands ###
