"""add archived field to software

Revision ID: 9683bd840dee
Revises: 4bd9bfb73f11
Create Date: 2025-07-14 12:42:05.362508

"""

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "9683bd840dee"
down_revision = "4bd9bfb73f11"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("software", schema=None) as batch_op:
        batch_op.add_column(sa.Column("archived", sa.Boolean(), nullable=True))

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("software", schema=None) as batch_op:
        batch_op.drop_column("archived")

    # ### end Alembic commands ###
